@@ -344,8 +344,8 @@ static int sox_checkformat(sox_format_t * ft)
 {
   ft->sox_errno = SOX_SUCCESS;
 
-  if (!ft->signal.rate) {
-    lsx_fail_errno(ft,SOX_EFMT,"sampling rate was not specified");
+  if (ft->signal.rate <= 0) {
+    lsx_fail_errno(ft, SOX_EFMT, "sample rate zero or negative");
     return SOX_EOF;
   }
   if (!ft->signal.precision) {
@@ -1197,7 +1197,7 @@ sox_get_format_fns(void)
   {
     lt_dlhandle lth = lt_dlopenext(file);
     const char *end = file + strlen(file);
-    const char prefix[] = "sox_fmt_";
+    const char prefix[] = "sox_ng_fmt_";
     char fnname[MAX_NAME_LEN];
     char *start = strstr(file, prefix);
 
