@@ -133,6 +133,12 @@ static int ft_enc(unsigned size, sox_encoding_t e)
   if (e == SOX_ENCODING_FLAC      && size == 16) return SF_FORMAT_PCM_16;
   if (e == SOX_ENCODING_FLAC      && size == 24) return SF_FORMAT_PCM_24;
   if (e == SOX_ENCODING_FLAC      && size == 32) return SF_FORMAT_PCM_32;
+#ifdef HAVE_SF_FORMAT_OGG
+  if (e == SOX_ENCODING_VORBIS)   return SF_FORMAT_VORBIS;
+#endif
+#ifdef HAVE_SF_FORMAT_MPEG
+  if (e == SOX_ENCODING_MP3)      return SF_FORMAT_MPEG_LAYER_III;
+#endif
   return 0; /* Bad encoding */
 }
 
@@ -170,6 +176,15 @@ static sox_encoding_t sox_enc(int ft_encoding, unsigned * size)
     case SF_FORMAT_DWVW_24  : *size = 24; return SOX_ENCODING_DWVW;
     case SF_FORMAT_DWVW_N   : *size =  0; return SOX_ENCODING_DWVWN;
     case SF_FORMAT_GSM610   : *size =  0; return SOX_ENCODING_GSM;
+#if HAVE_SF_FORMAT_OGG
+    case SF_FORMAT_VORBIS   : *size =  0; return SOX_ENCODING_VORBIS;
+#endif
+#if HAVE_SF_FORMAT_MPEG
+    case SF_FORMAT_MPEG_LAYER_II
+                            : *size = 16; return SOX_ENCODING_MP3;
+    case SF_FORMAT_MPEG_LAYER_III
+                            : *size = 16; return SOX_ENCODING_MP3;
+#endif
     default                 : *size =  0; return SOX_ENCODING_UNKNOWN;
   }
 }
@@ -186,9 +201,15 @@ static struct {
   { "snd",      SF_FORMAT_AU },
   { "caf",      SF_FORMAT_CAF },
   { "flac",     SF_FORMAT_FLAC },
-#ifdef HAVE_SNDFILE_1_0_18
-  { "wve",      SF_FORMAT_WVE },
-  { "ogg",      SF_FORMAT_OGG },
+  { "wve",      SF_FORMAT_WVE },  /* Probably broken before 1.0.18 */
+#ifdef HAVE_SF_FORMAT_OGG
+  { "ogg",      SF_FORMAT_OGG | SF_FORMAT_VORBIS },  /* From 1.0.16 */
+#endif
+#ifdef HAVE_SF_FORMAT_MPC2K
+  { "mpc2k",    SF_FORMAT_MPC2K },  /* From 1.0.25 */
+#endif
+#ifdef HAVE_SF_FORMAT_MPEG
+  { "mp3",      SF_FORMAT_MPEG | SF_FORMAT_MPEG_LAYER_III },  /* From 1.1.0 */
 #endif
   { "svx",      SF_FORMAT_SVX },
   { "8svx",     SF_FORMAT_SVX },
@@ -229,7 +250,9 @@ static sf_count_t vio_get_filelen(void *user_data)
 
 static sf_count_t vio_seek(sf_count_t offset, int whence, void *user_data)
 {
-    return lsx_seeki((sox_format_t *)user_data, (off_t)offset, whence);
+    if (lsx_seeki((sox_format_t *)user_data, (off_t)offset, whence))
+       return SOX_EINVAL;
+    return lsx_tell((sox_format_t *)user_data);
 }
 
 static sf_count_t vio_read(void *ptr, sf_count_t count, void *user_data)
@@ -304,9 +327,22 @@ static int start(sox_format_t * ft)
   /* Copy format info */
   if (subtype) {
     if (strcmp(ft->filetype, "sndfile") == 0)
-      sf->sf_info->format = name_to_format(ft->filename) | subtype;
+      sf->sf_info->format = name_to_format(ft->filename);
     else
-      sf->sf_info->format = name_to_format(ft->filetype) | subtype;
+      sf->sf_info->format = name_to_format(ft->filetype);
+
+    /* For linear types, name_to_format only returns the major format type
+     * and we need to OR in the bit depth */
+    switch (sf->sf_info->format) {
+    case SF_FORMAT_OGG | SF_FORMAT_VORBIS:
+#if HAVE_SF_FORMAT_MPEG
+    case SF_FORMAT_MPEG | SF_FORMAT_MPEG_LAYER_III:
+#endif
+      break;
+    default:
+      sf->sf_info->format |= subtype;
+      break;
+    }
   }
   sf->sf_info->samplerate = (int)ft->signal.rate;
   sf->sf_info->channels = ft->signal.channels;
@@ -443,7 +479,11 @@ static int startwrite(sox_format_t * ft)
       lsx_fail("cannot find a usable output encoding");
       return SOX_EOF;
     }
-    if ((sf->sf_info->format & SF_FORMAT_TYPEMASK) != SF_FORMAT_RAW)
+    if ((sf->sf_info->format & SF_FORMAT_TYPEMASK) != SF_FORMAT_RAW 
+#if HAVE_SF_FORMAT_MPEG
+        && (sf->sf_info->format & SF_FORMAT_TYPEMASK) != SF_FORMAT_MPEG
+#endif
+	)
       lsx_warn("cannot use desired output encoding, choosing default");
   }
 
