@@ -1431,15 +1431,15 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
 
     /* fact chunk (not PCM) */
     uint32_t dwFactSize=4;        /* length of the fact chunk */
-    uint32_t dwSamplesWritten=0;  /* windows doesnt seem to use this*/
+    uint64_t dwSamplesWritten=0;  /* windows doesnt seem to use this*/
 
     /* data chunk */
-    uint32_t  dwDataLength; /* length of sound data in bytes */
+    uint64_t  dwDataLength; /* length of sound data in bytes */
     /* end of variables written to header */
 
     /* internal variables, intermediate values etc */
     int bytespersample; /* (uncompressed) bytes per sample (per channel) */
-    long blocksWritten = 0;
+    uint64_t blocksWritten = 0;
     sox_bool isExtensible = sox_false;    /* WAVE_FORMAT_EXTENSIBLE? */
 
     if (ft->signal.channels > UINT16_MAX) {
@@ -1525,8 +1525,7 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
      * hint then write default value.  Also, use default value even
      * on header update if more then 32-bit length needs to be written.
      */
-    if ((!second_header && !ft->signal.length) || 
-        wav->numSamples > 0xffffffff) { 
+    if (!second_header && !ft->signal.length) {
         /* adjust for blockAlign */
         blocksWritten = MS_UNSPEC/wBlockAlign;
         dwDataLength = blocksWritten * wBlockAlign;
@@ -1539,7 +1538,7 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
     }
 
     if (wFormatTag == WAVE_FORMAT_GSM610)
-        dwDataLength = (dwDataLength+1) & ~1u; /* round up to even */
+        if (dwDataLength & 1) dwDataLength++; /* round up to even */
 
     if (wFormatTag == WAVE_FORMAT_PCM && (wBitsPerSample > 16 || wChannels > 2)
         && strcmp(ft->filetype, "wavpcm")) {
@@ -1554,7 +1553,10 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
         wRiffLength += (8+dwFactSize);
 
     /* dwAvgBytesPerSec <-- this is BEFORE compression, isn't it? guess not. */
-    dwAvgBytesPerSec = (double)wBlockAlign*ft->signal.rate / (double)wSamplesPerBlock + 0.5;
+    /* Round before dividing so that txw's 33333.3 doesn't become
+     * 33333 with 66667 byte rate.
+     */
+    dwAvgBytesPerSec = (wBlockAlign*lrint(ft->signal.rate)) / wSamplesPerBlock;
 
     /* figured out header info, so write it */
 
@@ -1639,16 +1641,27 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
         break;
     }
 
+    /* WAV files can't speciy lengths more than 4G samples or 4GB of data:
+     * warn and write UNSPEC instead of creating files of a random size. */
+    if (dwSamplesWritten > 0xffffffff) {
+        lsx_warn("length exceeds 4G samples: file may read truncated");
+	dwSamplesWritten = MS_UNSPEC;
+    }
+    if (dwDataLength > 0xffffffff) {
+        lsx_warn("length exceeds 4GB of data: file may read truncated");
+	dwDataLength = MS_UNSPEC;
+    }
+
     /* if not PCM, write the 'fact' chunk */
     if (isExtensible || wFormatTag != WAVE_FORMAT_PCM){
         if (lsx_writes(ft, "fact") ||
             lsx_writedw(ft,dwFactSize) ||
-            lsx_writedw(ft,dwSamplesWritten))
+            lsx_writedw(ft,(uint32_t)dwSamplesWritten))
 	        write_error();
     }
 
     if (lsx_writes(ft, "data") ||
-        lsx_writedw(ft, dwDataLength))               /* data chunk size */
+        lsx_writedw(ft, (uint32_t)dwDataLength))     /* data chunk size */
 	    write_error();
 
     if (!second_header) {
@@ -1659,13 +1672,13 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
                 dwAvgBytesPerSec, wBlockAlign, wBitsPerSample);
     } else {
         lsx_debug("Finished writing Wave file, %u data bytes %lu samples",
-                dwDataLength, (unsigned long)wav->numSamples);
+                (uint32_t)dwDataLength, (unsigned long)wav->numSamples);
         if (wFormatTag == WAVE_FORMAT_GSM610){
-            lsx_debug("GSM6.10 format: %li blocks %u padded samples %u padded data bytes",
-                    blocksWritten, dwSamplesWritten, dwDataLength);
+            lsx_debug("GSM6.10 format: %u blocks %u padded samples %u padded data bytes",
+                    (uint32_t)blocksWritten, (uint32_t)dwSamplesWritten, (uint32_t)dwDataLength);
             if (wav->gsmbytecount != dwDataLength)
                 lsx_warn("help ! internal inconsistency - data_written %u gsmbytecount %lu",
-                        dwDataLength, (unsigned long)wav->gsmbytecount);
+                        (uint32_t)dwDataLength, (unsigned long)wav->gsmbytecount);
 
         }
     }
